/*
 * NaryOperationAbstraction.hpp
 *
 *  Created on: 20. 8. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _NARY_OPERATION_ABSTRACTION_HPP_
#define _NARY_OPERATION_ABSTRACTION_HPP_

#include <alib/tuple>
#include <alib/memory>
#include <alib/array>

#include <abstraction/ValueOperationAbstraction.hpp>

#include <registry/Registry.h>

namespace abstraction {

template < class ReturnType, class ... ParamTypes >
class NaryOperationAbstraction : public ValueOperationAbstraction < ReturnType > {
protected:
	ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > m_params;

private:
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, unsigned index, bool move, bool checkInput ) override {
		if ( index >= m_params.size ( ) )
			throw std::invalid_argument ( "Parameter index " + ext::to_string ( index ) + " out of bounds.");

		if ( input == nullptr )
			return false;

		if ( checkInput && ! abstraction::CheckInput < ValueProvider < ParamTypes > ... >::checkInput ( input, index ) )
			return false;

		m_params [ index ].first = input;
		m_params [ index ].second = move;

		return true;
	}

	virtual bool detachInput ( unsigned index ) override {
		if ( index >= m_params.size ( ) )
			throw std::invalid_argument ( "Parameter index " + ext::to_string ( index ) + " out of bounds.");

		m_params [ index ].first = nullptr;
		m_params [ index ].second = false;

		return true;
	}

public:
	NaryOperationAbstraction ( ) {
		for ( unsigned i = 0; i < sizeof ... ( ParamTypes ); ++ i ) {
			m_params [ i ].first = nullptr;
			m_params [ i ].second = false;
		}
	}

	virtual bool inputsAttached ( ) const override {
		for ( const std::pair < std::shared_ptr < OperationAbstraction >, bool > & param : m_params )
			if ( ! param.first )
				return false;

		return true;
	}

	virtual bool eval ( ) override {
		if ( ! inputsAttached ( ) )
			return false;

		if ( this->cached ( ) )
			return true;

		for ( const std::pair < std::shared_ptr < OperationAbstraction >, bool > & param : m_params )
			if ( ! param.first->eval ( ) )
				return false;

		return this->run ( );
	}

	virtual unsigned numberOfParams ( ) const override {
		return sizeof ... ( ParamTypes );
	}

	virtual ext::type_index getParamTypeIndex ( unsigned index ) const override {
		return abstraction::ParamType < ParamTypes ... >::paramType ( index );
	}

	virtual ext::set < abstraction::ParamQualifiers::ParamQualifier > getParamTypeQualifiers ( unsigned index ) const override {
		return abstraction::ParamType < ParamTypes ... >::paramTypeQualifiers ( index );
	}

};

} /* namespace abstraction */

#endif /* _NARY_OPERATION_ABSTRACTION_HPP_ */
