/*
 * MinimizeDistinguishableStates.cpp
 *
 *  Created on: 27. 3. 2019
 *	  Author: Tomas Pecka
 */

#include "MinimizeDistinguishableStates.h"
#include <registration/AlgoRegistration.hpp>

namespace automaton {

namespace simplify {

auto MinimizeDistinguishableStatesDFA = registration::AbstractRegister < MinimizeDistinguishableStates, automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > >, const automaton::DFA < > & > ( MinimizeDistinguishableStates::minimize, "dfa" ).setDocumentation (
"Minimizes the given automaton using Distinguishable states method.\n\
\n\
@param dfa finite automaton to minimize.\n\
@return Minimal deterministic finite automaton equivalent to @p dfa" );

auto MinimizeDistinguishableStatesDFTA = registration::AbstractRegister < MinimizeDistinguishableStates, automaton::DFTA < DefaultSymbolType, DefaultRankType, ext::set < DefaultStateType > >, const automaton::DFTA < > & > ( MinimizeDistinguishableStates::minimize, "dfta" ).setDocumentation (
"Minimizes the given automaton using Distinguishable states method.\n\
\n\
@param dfta finite tree automaton to minimize.\n\
@return Minimal deterministic finite tree automaton equivalent to @p dfta" );

} /* namespace simplify */

} /* namespace automaton */
