/*
 * Trim.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "Trim.h"
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace simplify {

auto TrimCFG = registration::AbstractRegister < Trim, grammar::CFG < >, const grammar::CFG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimEpsilonFreeCFG = registration::AbstractRegister < Trim, grammar::EpsilonFreeCFG < >, const grammar::EpsilonFreeCFG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimGNF = registration::AbstractRegister < Trim, grammar::GNF < >, const grammar::GNF < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimCNF = registration::AbstractRegister < Trim, grammar::CNF < >, const grammar::CNF < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimLG = registration::AbstractRegister < Trim, grammar::LG < >, const grammar::LG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimLeftLG = registration::AbstractRegister < Trim, grammar::LeftLG < >, const grammar::LeftLG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimLeftRG = registration::AbstractRegister < Trim, grammar::LeftRG < >, const grammar::LeftRG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimRightLG = registration::AbstractRegister < Trim, grammar::RightLG < >, const grammar::RightLG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

auto TrimRightRG = registration::AbstractRegister < Trim, grammar::RightRG < >, const grammar::RightRG < > & > ( Trim::trim, "grammar" ).setDocumentation (
"Removes unproductive and unreachable symbols from the given grammar. Uses first the @sa UnproductiveSymbolsRemover and next @sa UnreachableSymbolsRemover in the process.\n\
\n\
@param grammar the context free grammar to trim\n\
@return the trimmed grammar equivalent to @p grammar" );

} /* namespace simplify */

} /* namespace grammar */

