/*
 * RawReaderRegistry.cpp
 *
 *  Created on: 21. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/RawReaderRegistry.hpp>
#include <alib/algorithm>
#include <alib/typeinfo>

namespace abstraction {

ext::map < std::string, std::unique_ptr < RawReaderRegistry::Entry > > & RawReaderRegistry::getEntries ( ) {
	static ext::map < std::string, std::unique_ptr < Entry > > readers;
	return readers;
}

std::shared_ptr < abstraction::OperationAbstraction > RawReaderRegistry::getAbstraction ( const std::string & type ) {
	for ( const std::pair < const std::string, std::unique_ptr < Entry > > & entry : getEntries ( ) ) {
		if ( ext::is_same_type ( entry.first, type ) )
			return entry.second->getAbstraction ( );
	}

	throw exception::CommonException ( "Entry " + type + " not available." );

}

} /* namespace abstraction */
