/*
 * RawWriterRegistry.hpp
 *
 *  Created on: 28. 8. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _RAW_WRITER_REGISTRY_HPP_
#define _RAW_WRITER_REGISTRY_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/set>
#include <alib/map>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class RawWriterRegistry {
public:
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

		virtual ~Entry ( ) {
		}
	};

private:
	template < class Param >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual ~EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;
	};

	static ext::map < std::string, std::unique_ptr < Entry > > & getEntries ( );

public:
	static void unregisterRawWriter ( std::string param ) {
		if ( ! getEntries ( ).erase ( param ) )
			throw std::invalid_argument ( "Entry " + param + " not registered." );
	}

	template < class ParamType >
	static void unregisterRawWriter ( ) {
		std::string param = ext::to_string < ParamType > ( );
		unregisterRawWriter ( std::move ( param ) );
	}

	template < class ParamType >
	static void registerRawWriter ( std::string param ) {
		if ( ! getEntries ( ).insert ( std::make_pair ( std::move ( param ), std::unique_ptr < Entry > ( new EntryImpl < ParamType > ( ) ) ) ).second )
			throw std::invalid_argument ( "Entry " + param + " already registered." );
	}

	template < class ParamType >
	static void registerRawWriter ( ) {
		std::string param = ext::to_string < ParamType > ( );
		registerRawWriter < ParamType > ( std::move ( param ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & param );
};

} /* namespace abstraction */

#include <abstraction/RawWriterAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > RawWriterRegistry::EntryImpl < Param >::getAbstraction ( ) const {
	return std::make_shared < abstraction::RawWriterAbstraction < const Param & > > ( );
}


} /* namespace abstraction */

#endif /* _RAW_WRITER_REGISTRY_HPP_ */
