/*
 * StringReaderRegistry.cpp
 *
 *  Created on: 21. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/StringReaderRegistry.hpp>

namespace abstraction {

ext::map < std::string, ext::list < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < StringReaderRegistry::Entry > > > > & StringReaderRegistry::getEntries ( ) {
	static ext::map < std::string, ext::list < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > > > readers;
	return readers;
}

std::shared_ptr < abstraction::OperationAbstraction > StringReaderRegistry::getAbstraction ( const std::string & group, const std::string & str ) {
	std::stringstream ss ( str );
	while ( isspace ( ss.peek ( ) ) )
		ss.get ( );

	auto lambda = [ & ] ( const std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > & entry ) {
		return entry.first ( ss );
	};

	const auto & entryIterator = getEntries ( ).find ( group );
	if ( entryIterator == getEntries ( ).end ( ) )
		throw exception::CommonException ( "Entry " + group + " not available." );

	const auto & entries = entryIterator->second;

	int pos = ss.tellg();

	typename ext::list < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > >::const_iterator callback = find_if ( entries.begin ( ), entries.end ( ), lambda );
	if ( callback == entries.end ( ) )
		throw exception::CommonException ( "No callback handling input found." );

	if ( pos != ss.tellg ( ) )
		throw exception::CommonException ( "First function of registered callback moved the stream." );

	return callback->second->getAbstraction ( );
}

} /* namespace abstraction */
