/*
 * StringReaderRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _STRING_READER_REGISTRY_HPP_
#define _STRING_READER_REGISTRY_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/map>
#include <alib/algorithm>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>
#include <factory/StringDataFactory.hpp>

namespace abstraction {

class StringReaderRegistry {
public:
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

		virtual ~Entry ( ) {
		}
	};

private:
	template < class Return >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual ~EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;
	};

	static ext::map < std::string, ext::list < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > > > & getEntries ( );

public:
	template < class Group >
	static void unregisterStringReader ( ext::list < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > >::const_iterator iter ) {
		std::string group = ext::to_string < Group > ( );

		auto & entry = getEntries ( ) [ group ];
		if ( ! ext::range_contains_iterator ( entry.begin ( ), entry.end ( ), iter ) )
			throw std::invalid_argument ( "Entry not found in " + group + "." );

		entry.erase ( iter );
		if ( entry.size ( ) == 0 )
			getEntries ( ).erase ( group );
	}

	template < class Group, class ReturnType >
	static ext::list < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > >::const_iterator registerStringReader ( ) {
		std::string group = ext::to_string < Group > ( );
		std::string returnType = ext::to_string < ReturnType > ( );

		auto & entry = getEntries ( ) [ group ];
		return entry.insert ( entry.end ( ), std::make_pair ( core::stringApi < ReturnType >::first, std::unique_ptr < Entry > ( new EntryImpl < ReturnType > ( ) ) ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & group, const std::string & data );
};

} /* namespace abstraction */

#include <abstraction/StringReaderAbstraction.hpp>

namespace abstraction {

template < class Return >
std::shared_ptr < abstraction::OperationAbstraction > StringReaderRegistry::EntryImpl < Return >::getAbstraction ( ) const {
	return std::make_shared < abstraction::StringReaderAbstraction < Return > > ( );
}

} /* namespace abstraction */

#endif /* _STRING_READER_REGISTRY_HPP_ */
