/*
 * XmlContainerParserRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _XML_CONTAINER_REGISTRY_HPP_
#define _XML_CONTAINER_REGISTRY_HPP_

#include <alib/functional>
#include <alib/memory>
#include <alib/list>
#include <alib/string>
#include <alib/set>
#include <alib/map>
#include <alib/typeinfo>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class XmlContainerParserRegistry {
public:
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

		virtual ~Entry ( ) {
		}
	};

private:
	template < class Params >
	class SetEntryImpl : public Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;

		virtual ~SetEntryImpl ( ) {
		}
	};

	static ext::map < std::string, ext::list < ext::pair < std::string, std::unique_ptr < Entry > > > > & getEntries ( );

public:
	static void unregisterSet ( std::string param ) {
		std::string container = "Set";

		auto & group = getEntries ( ) [ container ];
		auto iter = find_if ( group.begin ( ), group.end ( ), [ & ] ( const ext::pair < std::string, std::unique_ptr < Entry > > & entry ) {
				return entry.first == param;
				} );
		if ( iter == group.end ( ) )
			throw exception::CommonException ( "Callback for " + param + " in container " + container + " not registered." );

		group.erase ( iter );
	}

	template < class ParamTypes >
	static void unregisterSet ( ) {
		std::string param = ext::to_string < typename std::decay < ParamTypes >::type > ( );
		unregisterSet ( param );
	}

	template < class ParamTypes >
	static void registerSet ( std::string param ) {
		std::string container = "Set";

		auto & group = getEntries ( ) [ container ];
		for ( const ext::pair < std::string, std::unique_ptr < Entry > > & entry : group )
			if ( entry.first == param )
				throw exception::CommonException ( "Callback for " + param + " in container " + container + " already registered." );

		group.insert ( group.end ( ), ext::make_pair ( param, std::make_unique < SetEntryImpl < ParamTypes > > ( ) ) );
	}

	template < class ParamTypes >
	static void registerSet ( ) {
		std::string param = ext::to_string < typename std::decay < ParamTypes >::type > ( );
		registerSet < ParamTypes > ( param );
	}

	static bool hasAbstraction ( const std::string & container );

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & container, const std::string & type );

	static ext::set < std::string > listOverloads ( const std::string & algorithm );

	static ext::set < std::string > list ( );
};

} /* namespace abstraction */

#include <abstraction/XmlParserAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > XmlContainerParserRegistry::SetEntryImpl < Param >::getAbstraction ( ) const {
	return std::make_shared < abstraction::XmlParserAbstraction < ext::set < Param > > > ( );
}

} /* namespace abstraction */

#endif /* _XML_CONTAINER_REGISTRY_HPP_ */
